/* 
 * $RCSfile: idmap_special.h,v $
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 *
 * $Log: idmap_special.h,v $
 * Revision 1.1  1994/06/08  23:15:11  davidm
 * Initial revision
 *
 */

/*
 * This generates special hash/comparison/assignment functions
 * for keys of size 2, 4, 6, 8, and 16 bytes.   It also generates
 * the generic functions that can deal with any size.
 */
#ifdef XK_USE_INLINE
#  define INLINE	__inline__
#else
#  define INLINE
#endif

#ifndef CONST
# ifdef __STDC__
#  define CONST const
# else
#  define CONST
# endif
#endif

static INLINE int	hash2 (void *, int);
static INLINE int	bcmp2 (const char *, const char *);
static INLINE void	bcpy2 (char *, const char *);

static INLINE int	hash4 (void *, int);
static INLINE int	bcmp4 (const char *, const char *);
static INLINE void	bcpy4 (char *, const char *);

static INLINE int	hash6 (void *, int);
static INLINE int	bcmp6 (const char *, const char *);
static INLINE void	bcpy6 (char *, const char *);

static INLINE int	hash8 (void *, int);
static INLINE int	bcmp8 (const char *, const char *);
static INLINE void	bcpy8 (char *, const char *);

static INLINE int	hash10 (void *, int);
static INLINE int	bcmp10 (const char *, const char *);
static INLINE void	bcpy10 (char *, const char *);

static INLINE int	hash12 (void *, int);
static INLINE int	bcmp12 (const char *, const char *);
static INLINE void	bcpy12 (char *, const char *);

static INLINE int	hash16 (void *, int);
static INLINE int	bcmp16 (const char *, const char *);
static INLINE void	bcpy16 (char *, const char *);

/*
 * Generic compare/copy routines:
 */
#define GEN_CMP_BYTES(s1, s2, n)	(!bcmp(s1, s2, n))
#define GEN_CPY_BYTES(s1, s2, n)	(bcopy(s2, s1, n))

/*
 * Macro Cx(i,o) expands to code that applies binary operator
 * "o" to an 8*x bit integers at addresses s1+i and s2+i.  s1
 * and s2 must be of type "char *".
 */
#define C2(i,o)		(*(xk_int16*)((s1)+(i)) o *(xk_int16*)((s2)+(i)))
#define C4(i,o)		(*(xk_int32*)((s1)+(i)) o *(xk_int32*)((s2)+(i)))
#ifdef xk_int64
# define C8(i,o)	(*(xk_int64*)((s1)+(i)) o *(xk_int64*)((s2)+(i)))
#endif

/*
 * Macro C(t,e,gen,n) expands to code that evaluates expression
 * "e" if both, "s1" and "s2" are aligned to type "t".  If either
 * pointer is not properly aligned, the expression "gen(s1,s2,n)"
 * is evaluated instead.
 */
#define C(t,e,gen,n) \
  (XK_ALIGNED((s1),t) && XK_ALIGNED((s2),t)) ? (e) : gen((s1),(s2),n)

/***************** keysize = 2 bytes *****************/

#define KEYSIZE 2
#define BINDNAME m2bind
#define REMOVENAME m2remove
#define RESOLVENAME m2resolve
#define UNBINDNAME m2unbind

static INLINE int
hash2(K, tblSize)
     VOID  *K;
     int tblSize;
{
    u_int h;
    u_char h0, h1;

    h0 = *(u_char *)K;
    h1 = *((u_char *)K+1);
    /* tblSize must be power of two: */
    xAssert((tblSize & (tblSize - 1)) == 0);
    h = (((h0<<2) + h1) ^ ((h1<<2) + h0)) & (tblSize - 1);
    xTrace2(idmap, TR_DETAILED, "2 byte key optimized hash of %x is %d",
	    h0|(h1<<8), h);
    return h;
} /* hash2 */

static INLINE int
bcmp2(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
    return C(xk_int16, (C2(0,==)), GEN_CMP_BYTES, 2);
} /* bcmp2 */

static INLINE void
bcpy2(s1, s2)
     char *s1;
     CONST char *s2;
{
    C(xk_int16, (C2(0,=)), GEN_CPY_BYTES, 2);
} /* bcpy2 */

#define HASH(K, tblSize, keySize)	hash2(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp2(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy2(s1, s2)

#include "idmap_templ.c"

/***************** keysize = 4 bytes *****************/

#define KEYSIZE 4
#define BINDNAME m4bind
#define REMOVENAME m4remove
#define RESOLVENAME m4resolve
#define UNBINDNAME m4unbind

static INLINE int
hash4(K, tblSize)
     VOID *K;
     int tblSize;
{
    register u_char	h0=*(u_char *)K,
                        h1=*((u_char *)K+1),
                        h2=*((u_char *)K+2),
			h3=*((u_char *)K+3);
    u_int h;
#ifdef XK_DEBUG
    u_long k = h0|((u_long) h1<<8)|((u_long) h2<<16)|((u_long) h3<<24);
#endif

    /* tblSize must be power of two: */
    xAssert((tblSize & (tblSize - 1)) == 0);
    h = ((h0+(h2<<2)) ^ (h1+(h3<<1)) ^ (h2+(h0<<1)) ^ ((h3+h1)<<2)) 
      & (tblSize - 1);

    xTrace4(idmap, TR_DETAILED,
	    "4 byte key optimized hash of %lx (at %lx) is %d in table size %d",
	    k, (unsigned long)K, h, tblSize);
    return h;
} /* hash4 */

static INLINE int
bcmp4(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
    return C(xk_int32, (C4(0,==)), GEN_CMP_BYTES, 4);
} /* bcmp4 */

static INLINE void
bcpy4(s1, s2)
     char *s1;
     CONST char *s2;
{
    C(xk_int32, (C4(0,=)), GEN_CPY_BYTES, 4);
} /* bcpy4 */

#define HASH(K, tblSize, keySize)	hash4(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp4(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy4(s1, s2)

/***************** keysize = 6 bytes *****************/

#include "idmap_templ.c"

#define KEYSIZE 6
#define BINDNAME m6bind
#define REMOVENAME m6remove
#define RESOLVENAME m6resolve
#define UNBINDNAME m6unbind

static INLINE int
hash6(K, tblSize)
     VOID *K;
     int tblSize;
{
    u_int	h = 0;
    register    u_char	*k = (u_char *)K;

    h = hash4(k,tblSize) ^ hash2(k+4,tblSize);

    xTrace1(idmap, TR_FULL_TRACE, "6 byte key optimized hash returns %d", h);
    return h;
} /* hash6 */

static INLINE int
bcmp6(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
    return C(xk_int32, (C4(0,==) && C2(4,==)), GEN_CMP_BYTES, 6);
} /* bcmp6 */

static INLINE void
bcpy6(s1, s2)
     char *s1;
     CONST char *s2;
{
    C(xk_int32, (C4(0,=), C2(4,=)), GEN_CPY_BYTES, 6);
} /* bcpy6 */

#define HASH(K, tblSize, keySize) 	hash6(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp6(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy6(s1, s2)

#include "idmap_templ.c"

/***************** keysize = 8 bytes *****************/

#define KEYSIZE 8
#define BINDNAME m8bind
#define REMOVENAME m8remove
#define RESOLVENAME m8resolve
#define UNBINDNAME m8unbind

static INLINE int
hash8(K, tblSize)
     VOID *K;
     int tblSize;
{
    u_int	h;
    register    u_char	*k = (u_char *)K;

    h = hash4(k,tblSize) ^ hash4(k+4,tblSize);

    xTrace1(idmap, TR_FULL_TRACE, "8 byte key optimized hash returns %d", h);
    return h;
} /* hash8 */

static INLINE int
bcmp8(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    return C(xk_int64, (C8(0,==)), GEN_CMP_BYTES, 8);
#else
    return C(xk_int32, (C4(0,==) && C4(4,==)), GEN_CMP_BYTES, 8);
#endif
} /* bcmp8 */

static INLINE void
bcpy8(s1, s2)
     char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    C(xk_int64, (C8(0,=)), GEN_CPY_BYTES, 8);
#else
    C(xk_int32, (C4(0,=), C4(4,=)), GEN_CPY_BYTES, 8);
#endif
} /* bcpy8 */

#define HASH(K, tblSize, keySize)	hash8(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp8(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy8(s1, s2)

#include "idmap_templ.c"

/***************** keysize = 10 bytes *****************/

#define KEYSIZE 10
#define BINDNAME m10bind
#define REMOVENAME m10remove
#define RESOLVENAME m10resolve
#define UNBINDNAME m10unbind

static INLINE int
hash10(K, tblSize)
     VOID *K;
     int tblSize;
{
    u_int	h;
    register    u_char	*k = (u_char *)K;

    h = hash4(k,tblSize) ^ hash4(k+4,tblSize) ^ hash2(k+8,tblSize);

    xTrace1(idmap, TR_FULL_TRACE, "10 byte key optimized hash returns %d", h);
    return h;
} /* hash10 */

static INLINE int
bcmp10(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    return C(xk_int64, (C8(0,==) && C2(8,==)), GEN_CMP_BYTES, 10);
#else
    return C(xk_int32, (C4(0,==) && C4(4,==) && C2(8,==)), GEN_CMP_BYTES, 10);
#endif
} /* bcmp10 */

static INLINE void
bcpy10(s1, s2)
     char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    C(xk_int64, (C8(0,=), C2(8,=)), GEN_CPY_BYTES, 10);
#else
    C(xk_int32, (C4(0,=), C4(4,=), C2(8,=)), GEN_CPY_BYTES, 10);
#endif
} /* bcpy10 */

#define HASH(K, tblSize, keySize) 	hash10(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp10(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy10(s1, s2)

#include "idmap_templ.c"

/***************** keysize = 12 bytes *****************/

#define KEYSIZE 12
#define BINDNAME m12bind
#define REMOVENAME m12remove
#define RESOLVENAME m12resolve
#define UNBINDNAME m12unbind

static INLINE int
hash12(K, tblSize)
    VOID *K;
    int tblSize;
{
    u_int	h;
    register    u_char	*k = (u_char *)K;

    h = hash4(k,tblSize) ^ hash4(k+4,tblSize) ^ hash4(k+8,tblSize);
    xTrace1(idmap, TR_FULL_TRACE, "12 byte key optimized hash returns %d", h);
    return h;
} /* hash12 */

static INLINE int
bcmp12(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    return C(xk_int64, (C8(0,==) && C4(8,==)), GEN_CMP_BYTES, 12);
#else
    return C(xk_int32, (C4(0,==) && C4(4,==) && C4(8,==)), GEN_CMP_BYTES, 12);
#endif
} /* bcmp12 */

static INLINE void
bcpy12(s1, s2)
     char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    C(xk_int64, (C8(0,=), C4(8,=)), GEN_CPY_BYTES, 12);
#else
    C(xk_int32, (C4(0,=), C4(4,=), C4(8,=)), GEN_CPY_BYTES, 12);
#endif
} /* bcpy12 */

#define HASH(K, tblSize, keySize)	hash12(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp12(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy12(s1, s2)

/***************** keysize = 16 bytes *****************/

#include "idmap_templ.c"

#define KEYSIZE 16
#define BINDNAME m16bind
#define REMOVENAME m16remove
#define RESOLVENAME m16resolve
#define UNBINDNAME m16unbind

static INLINE int
hash16(K, tblSize)
    VOID *K;
    int tblSize;
{
    u_int	h;
    register    u_char	*k = (u_char *)K;

    h = hash4(k,tblSize) ^ hash4(k+4,tblSize) ^ hash4(k+8,tblSize)
        ^ hash4(k+12,tblSize);
    xTrace1(idmap, TR_FULL_TRACE, "16 byte key optimized hash returns %d", h);
    return h;
} /* hash16 */

static INLINE int
bcmp16(s1, s2)
     CONST char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    return C(xk_int64, (C8(0,==) && C8(8,==)), GEN_CMP_BYTES, 16);
#else
    return C(xk_int32, (C4(0,==) && C4(4,==) && C4(8,==) && C4(12,==)),
	     GEN_CMP_BYTES, 16);
#endif
} /* bcmp16 */

static INLINE void
bcpy16(s1, s2)
     char *s1;
     CONST char *s2;
{
#ifdef xk_int64
    C(xk_int64, (C8(0,=), C8(8,=)), GEN_CPY_BYTES, 16);
#else
    C(xk_int32, (C4(0,=), C4(4,=), C4(8,=), C4(12,=)), GEN_CPY_BYTES, 16);
#endif
} /* bcpy16 */

#define HASH(K, tblSize, keySize)	hash16(K, tblSize)
#define COMPBYTES(s1,s2)		bcmp16(s1, s2)
#define COPYBYTES(s1,s2) 		bcpy16(s1, s2)

#include "idmap_templ.c"

/***************** generic operations *****************/

static int
generichash(key, tableSize, keySize)
    char *key;
    int tableSize;
    int keySize;
{
    unsigned int hash = 0;
    int howmanywords = keySize / 4;

    xTrace2(idmap, TR_FULL_TRACE,
	    "generic idmap hash -- tableSize %d, keySize %d",
	    tableSize, keySize);
    while (howmanywords--) {
	hash ^= hash4(key, tableSize);
	key += 4;
    }
    if (keySize & 0x2) {
	hash ^= hash2(key, tableSize);
	key += 2;
    }
    if (keySize & 0x1) {
	hash ^=  *(u_char *)key;
    }
    /* tableSize must be power of two: */
    xAssert((tableSize & (tableSize - 1)) == 0);
    hash = hash & (tableSize - 1);
    xTrace1(idmap, TR_FULL_TRACE, "generic hash returns %d", hash);
    return hash;
} /* generichash */

#define KEYSIZE generic
#define BINDNAME mgenericbind
#define REMOVENAME mgenericremove
#define RESOLVENAME mgenericresolve
#define UNBINDNAME mgenericunbind
#define HASH(K, tblSize, keySize) generichash(K, tblSize, keySize)
#define GENHASH(K, T, keySize) generichash(K, tblSize, keySize)
#define COMPBYTES(s1, s2) (!bcmp(s1, s2, table->keySize))
#define COPYBYTES(s1, s2) bcopy(s2, s1, table->keySize)

#include "idmap_templ.c"
