/*
 * @COPYRIGHT@
 * 
 * x-kernel v3.2
 * 
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 * 
 * @COPYRIGHT@
 *
 * $RCSfile: alpha_prom.h,v $
 *
 * HISTORY
 * $Log: alpha_prom.h,v $
 * Revision 1.2  1994/06/11  00:02:52  menze
 * Added copyright notice
 *
 * Revision 1.1  1994/04/19  03:00:59  davidm
 * Initial revision
 *
 */
/*
 * The following is based on information published in
 * DEC 3000 300/400/500/600/800 Models, System Programmer's Manual,
 * DEC Part #: EK-D3SYS-PM.A01.
 */
#ifndef alpha_prom_h
#define alpha_prom_h

#include <machine/rpb.h>

/*
 * NV---Nonvolatile.  The last value saved by system software or set
 * by console commands is preserved accross system initializations,
 * cold bootstraps, and long power outages.
 *
 * W---Warm nonvolatile.  The last value set by system software is
 * preserved across warm bootstraps and restarts.
 *
 * RO---Read-only.  The variable cannot be modified by system software
 * or console commands.
 */
#define PROM_E_AUTO_ACTION	0x01	/* NV W    */
#define PROM_E_BOOT_DEV		0x02	/*    W    */
#define PROM_E_BOOTCMD_DEV	0x03	/* NV W    */
#define PROM_E_BOOTED_DEV	0x04	/*      RO */
#define PROM_E_BOOT_FILE	0x05	/* NV W    */
#define PROM_E_BOOTED_FILE	0x06	/*      RO */
#define PROM_E_BOOT_OSFLAGS	0x07	/* NV W    */
#define PROM_E_BOOTED_OSFLAGS	0x08	/*      RO */
#define PROM_E_BOOT_RESET	0x09	/* NV W    */
#define PROM_E_DUMP_DEV		0x0a	/* NV W    */
#define PROM_E_ENABLE_AUDIT	0x0b	/* NV W    */
#define PROM_E_LICENSE		0x0c	/* NV   RO */
#define PROM_E_CHAR_SET		0x0d	/* NV W    */
#define PROM_E_LANGUAGE		0x0e	/* NV W    */
#define PROM_E_TTY_DEV		0x0f	/* NV W RO */

/*
 * DISPATCH return value definition.  The exact layout of the return value
 * is call-specific, but "retval" and "status" are always defined.
 */
typedef union {
    struct {
	unsigned retval	: 32;
	unsigned unit	: 8;
	unsigned mbz	: 8;
	unsigned error	: 13;	/* device-specific error status */
	unsigned status	: 3;	/* 0 == SUCCESS, other values: call-specific */
    } u;
    unsigned long bits;
} PROM_Result;

typedef union {
    struct {
	unsigned rev	: 8;	/* revision */
	unsigned vers	: 8;	/* version */
	unsigned flavor	: 8;	/* flavor (OSF/1, VMS, etc.) */
    } u;
    unsigned long bits;
} PALcode_Revision;

extern struct bootpcb	boot_pcb;

/*
 * Return type is actually of type PROMResult, but GCC seems
 * to treat a union of 8 bytes differently from a long.
 */
extern long	prom_dispatcher (long code, ...);

extern long	prom_switch_to_osf_pal_code (void);

#endif /* alpha_prom_h */
